@@ -1,19 +1,15 @@
 """Compatability code for older python version."""
+from __future__ import annotations
 
 try:
     from importlib.metadata import version
 except ImportError:  # pragma: py-gte-38
-    from importlib_metadata import version
-
-try:
-    from typing import Literal
-except ImportError:  # pragma: py-gte-38
-    from typing_extensions import Literal  # type: ignore[misc]
+    from importlib_metadata import version  # type: ignore[import,no-redef]
 
 try:
     from typing import Protocol
 except ImportError:  # pragma: py-gte-38
-    from typing_extensions import Protocol  # type: ignore[misc]
+    from typing_extensions import Protocol  # type: ignore[assignment]
 
 try:
     from typing import TypedDict
@@ -21,4 +17,4 @@ except ImportError:  # pragma: py-gte-38
     from typing_extensions import TypedDict
 
 
-__all__ = ["Literal", "Protocol", "TypedDict", "version"]
+__all__ = ["Protocol", "TypedDict", "version"]
